import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:our_wedding_sdk/src/model/invite.dart';
import 'package:our_wedding_sdk/src/utils/utils_http.dart';

class InviteHttp extends UtilsHttp{
  static final InviteHttp _inviteHttp = new InviteHttp._internal();

  factory InviteHttp() {
    return _inviteHttp;
  }

  InviteHttp._internal();

  Future<Invite> getInvite({@required String idInvite, @required String token}) async{
    final response = await http
        .get(UtilsHttp.serverIp + 'invite/$idInvite',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        Invite ret = Invite.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      return null;
    }
  }

  Future<Invite> postInvite({@required Invite invite, @required String token}) async {
    final response = await http
        .post(UtilsHttp.serverIp + 'invite/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(invite),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 201 && jsonVar != null) {
        Invite ret = Invite.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      return null;
    }
  }

  deleteInvite({@required String idInvite, @required String token}) async {
    final response = await http
        .delete(UtilsHttp.serverIp + 'invite/$idInvite',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    return response.statusCode;
  }

}