import 'dart:collection';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:our_wedding_sdk/src/model/invite.dart';
import 'package:our_wedding_sdk/src/model/user.dart';
import 'package:our_wedding_sdk/src/utils/utils_http.dart';

class UserHttp extends UtilsHttp{
  static final UserHttp _userHttp = new UserHttp._internal();

  HashMap<int, User> _userHash = HashMap();

  factory UserHttp() {
    return _userHttp;
  }

  UserHttp._internal();

  Future<User> postUser({@required user, @required token}) async {
    final response = await http
        .post(UtilsHttp.serverIp + 'user/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(user),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 201 && jsonVar != null) {
        User ret = User.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      return null;
    }
  }

  Future<User> putUser({@required User user, String token}) async{
    User ret = User();
    var idUser = user.idUser;
    final response = await http
        .put(UtilsHttp.serverIp + 'user/$idUser',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(user),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        ret = User.fromJson(jsonVar);
      }
    } catch (e) {

    }
    return ret;
  }

  Future<List<User>> getAllUsers({@required String token}) async {
    List<User> users = [];
    final response = await http
        .get(UtilsHttp.serverIp + 'user/',
        headers: {
      "Content-Type": "application/json",
      "authorization": "Basic " + token
    });
    try {
      var jsonVar = json.decode(response.body);
      List value = jsonVar['value'];
      users = value.map((userJson) => User.fromJson(userJson)).toList();
      for (int i = 0; i < users.length; i++) {
        User user = users.elementAt(i);
        _userHash[user.idUser] = user;
      }
    } catch (e) {
      return users;
    }
    users.sort((user1, user2) => user1.userName.compareTo(user2.userName));
    return users;
  }

  Future<User> loginUser({@required String email, @required token}) async{
    final response = await http
        .get(UtilsHttp.serverIp + 'user/email/$email/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      var value = jsonVar['value'];
      var userJson = value[0];
      if (response.statusCode == 200 && userJson != null) {
        User user = User.fromJson(userJson);
        //user.token = token;
        return user;
      }
    } catch (e) {
      return null;
    }
  }

  Future<User> getUser({int idUser, String token}) async {
    final response = await http
        .get(UtilsHttp.serverIp + 'user/$idUser',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        User user = User.fromJson(jsonVar);
        return user;
      }
    } catch (e) {
      return null;
    }
  }

  Future<int> deleteUser({int deleteUser, int adminUser, String token}) async {
    final response = await http
        .delete(UtilsHttp.serverIp + 'user/?deleteUser=$deleteUser&adminUser=$adminUser',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    return response.statusCode;
  }

  Future<List<User>> getInviteUsers({@required Invite invite, @required String token}) async {
    String idInvite = invite.idInvite;
    List<User> users = [];
    final response = await http
        .get(UtilsHttp.serverIp + 'user/invite?invite=$idInvite',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      List value = jsonVar['value'];
      users = value.map((userJson) => User.fromJson(userJson)).toList();
      for (int i = 0; i < users.length; i++) {
        User user = users.elementAt(i);
        _userHash[user.idUser] = user;
      }
    } catch (e) {
      return users;
    }
    users.sort((user1, user2) => user1.userName.compareTo(user2.userName));
    return users;
  }

}